// Modules to control application life and create native browser window
const { app, BrowserWindow, dialog, session, shell, Menu, Tray, nativeImage } = require('electron')
const path = require('path')
const ipcMain = require('electron').ipcMain;

var log4js = require("log4js");
var logger = log4js.getLogger();
log4js.configure({
    appenders: { launch: { type: "file", filename: "launch.log" } },
    categories: { default: { appenders: ["launch"], level: "trace" } }
  });

// QA
// const API_URL="https://api-qa2.iartest.com/api";
// const IAR_MEMBER_URL="https://www-qa2.iartest.com/v3/agency/def.aspx?kv2="
// const IAR_DISPATCHER_URL="https://www-qa2.iartest.com/v3/Pages/DispatcherScreen.aspx?kv2="
// const ENCRYPT_DATA_SECRET_KEY="wek2m7wgby"

// UAT
// const IAR_MEMBER_URL="https://www.iartest.com/v3/agency/def.aspx?kv2=";
// const IAR_DISPATCHER_URL="https://www.iartest.com/v3/Pages/DispatcherScreen.aspx?kv2=";
// const ENCRYPT_DATA_SECRET_KEY="wek2m7wgby"

// PROD
const API_URL="https://api.iamresponding.com/api";
const IAR_MEMBER_URL="https://www.iamresponding.com/v3/agency/def.aspx?kv2=";
const IAR_DISPATCHER_URL="https://www.iamresponding.com/v3/Pages/DispatcherScreen.aspx?kv2=";
const ENCRYPT_DATA_SECRET_KEY="Ygv9VR5WNydoSIP"

let mainWindow = null;
function createWindow() {
  logger.debug('INFO > main > Start window')
  // Delete all data inc. cookies
  // session.defaultSession.clearStorageData([], (data) => {})

  // Create the browser window.
  mainWindow = new BrowserWindow({
    width: 700,
    height: 640,
    resizable: false,
    webPreferences: {
      preload: path.join(__dirname, 'preload.js'),
      enableRemoteModule: true,
      nodeIntegration: true,
      contextIsolation: false
    },
    icon: './Images/Icon_Square.ico',
    backgroundColor: '#ffffff'
  })

  ipcMain.on('hide-window',()=>{
    //if mainWindow is the window object
    mainWindow.hide();
  })

  ipcMain.on('close-window',()=>{
    //if mainWindow is the window object
    mainWindow.close();
  })

  // and load the index.html of the app.
  try {
    logger.debug('INFO > main > Loading HTML')
    mainWindow.loadFile('index.html')
    mainWindow.setMenu(null)
    logger.debug('INFO > main > End window')
    //mainWindow.webContents.openDevTools()
  } catch (error) {
    logger.debug('ERROR > main > ' + JSON.stringify(error))
  }
}

// This method will be called when Electron has finished
// initialization and is ready to create browser windows.
// Some APIs can only be used after this event occurs.
let tray = null
app.whenReady().then(() => {
  try {
    logger.debug('INFO > main > Ready to create window')
    createWindow()

    logger.debug('INFO > main > create system try')
    const iconPath = path.join(__dirname, 'Images/Icon_IaR.png')
    const nimage = nativeImage.createFromPath(iconPath)
    tray = new Tray(nimage)
    const contextMenu = Menu.buildFromTemplate([
      { label: 'Change Settings', type: 'normal', click: () => {
        mainWindow.show();
        mainWindow.focus();
      } },
      { label: 'Exit Application', type: 'normal', click: () => { app.quit() } }
    ])
    tray.setToolTip('IamResponding')
    tray.setContextMenu(contextMenu)

    app.on('activate', function () {
      // On macOS it's common to re-create a window in the app when the
      // dock icon is clicked and there are no other windows open.
      if (BrowserWindow.getAllWindows().length === 0) createWindow()
    })
    logger.debug('INFO > main > End create window')
  } catch (error) {
    logger.debug('ERROR > main > ' + JSON.stringify(error))
  }
})

// Quit when all windows are closed, except on macOS. There, it's common
// for applications and their menu bar to stay active until the user quits
// explicitly with Cmd + Q.
app.on('window-all-closed', function () {
  if (process.platform !== 'darwin') app.quit()
})

// In this file you can include the rest of your app's specific main process
// code. You can also put them in separate files and require them here.

// this should be placed at top of main.js to handle setup events quickly
if (handleSquirrelEvent()) {
  // squirrel event handled and app will exit in 1000ms, so don't do anything else
  return;
}

function handleSquirrelEvent() {
  if (process.argv.length === 1) {
    return false;
  }

  const ChildProcess = require('child_process');
  const path = require('path');

  const appFolder = path.resolve(process.execPath, '..');
  const rootAtomFolder = path.resolve(appFolder, '..');
  const updateDotExe = path.resolve(path.join(rootAtomFolder, 'Update.exe'));
  const exeName = path.basename(process.execPath);

  const spawn = function(command, args) {
    let spawnedProcess, error;

    try {
      spawnedProcess = ChildProcess.spawn(command, args, {detached: true});
    } catch (error) {
      logger.debug('main - e')
      logger.debug(error)
    }

    return spawnedProcess;
  };

  const spawnUpdate = function(args) {
    return spawn(updateDotExe, args);
  };

  const squirrelEvent = process.argv[1];
  switch (squirrelEvent) {
    case '--squirrel-install':
    case '--squirrel-updated':
      // Optionally do things such as:
      // - Add your .exe to the PATH
      // - Write to the registry for things like file associations and
      //   explorer context menus

      setTimeout(app.quit, 1000);
      return true;

    case '--squirrel-uninstall':
      // Undo anything you did in the --squirrel-install and
      // --squirrel-updated handlers

      setTimeout(app.quit, 1000);
      return true;

    case '--squirrel-obsolete':
      // This is called on the outgoing version of your app before
      // we update to the new version - it's the opposite of
      // --squirrel-updated

      app.quit();
      return true;
  }
};
