const electron = require('electron');
const ipc = require('electron').ipcRenderer;
const shell = require('electron').shell;
const request = require('request');
const AutoLaunch = require('auto-launch');
const encrypt_module = require('./encrypt')
const detectBrowsers = require('detect-browsers');
const fs = require('fs');
const jwt = require('jsonwebtoken')
const childProcess = require('child_process');
var internetAvailable = require("internet-available");
const jwt_decode = require('jwt-decode')
const superagent = require('superagent')
var path = require('path');
const getMAC = require('getmac').default;
const os = require("os");
var log4js = require("log4js");
var logger = log4js.getLogger();
log4js.configure({
    appenders: { launch: { type: "file", filename: "launch.log" } },
    categories: { default: { appenders: ["launch"], level: "trace" } }
  });

let btnLaunch = document.getElementById('btnLaunch')
let btnCancel = document.getElementById('btnCancel')
let closeModal = document.getElementsByClassName("close")[0];
let modal = document.getElementById("myModal");
let sliderOff = document.getElementById('sliderOff');
let sliderOn = document.getElementById('sliderOn');
let offKiosk = document.getElementById('offKiosk')
let onKiosk = document.getElementById('onKiosk')
let isAutoLauncher = true
let enableKioskMode = true
const version = "2.0.0";

const currentOS = process.platform;
//These two lines are for create installer:
const ca_cert = fs.readFileSync(path.join(path.dirname(__dirname), './app/cacert.pem'))

//These lines are for debug mode:
//const ca_cert = fs.readFileSync('./cacert.pem')

// # # QA
// const API_URL="https://api-qa2.iartest.com/api"
// const IAR_MEMBER_URL="https://www-qa2.iartest.com/v3/agency/def.aspx?kv2="
// const IAR_DISPATCHER_URL="https://www-qa2.iartest.com/v3/Pages/DispatcherScreen.aspx?kv2="
// const IAR_SECURITY = "https://auth-qa.iartest.com/login/autolauncher/redirect?token=";
// const ENCRYPT_DATA_SECRET_KEY="wek2m7wgby"
const API_GATEWAY="https://api-gateway-qa.iartest.com/api/v1/";

// # UAT
// const API_URL="https://api.iartest.com/api/";
// const IAR_MEMBER_URL="https://www.iartest.com/v3/agency/def.aspx?kv2=";
// const IAR_DISPATCHER_URL="https://www.iartest.com/v3/Pages/DispatcherScreen.aspx?kv2=";
// const IAR_SECURITY = "https://auth.iartest.com/login/autolauncher/redirect?token=";
// # ENCRYPT_DATA_SECRET_KEY="Ygv9VR5WNydoSIP"

// # PRD
const API_URL="https://api.iamresponding.com/api";
const IAR_MEMBER_URL="https://www.iamresponding.com/v3/agency/def.aspx?kv2=";
const IAR_DISPATCHER_URL="https://www.iamresponding.com/v3/Pages/DispatcherScreen.aspx?kv2=";
const IAR_SECURITY = "https://auth.iamresponding.com/login/autolauncher/redirect?token=";
const ENCRYPT_DATA_SECRET_KEY="Ygv9VR5WNydoSIP"
//const API_GATEWAY="https://api-gateway.iamresponding.com/api/v1/";

document.getElementById('internetUnavailable').style.display = "none"
document.getElementById('app_version').innerHTML = "v." + version;

if(currentOS == "win32") {
const autoupdateApp = new AutoLaunch({
    name: 'IaRAutoupdate',
    path: 'C:\\Program Files (x86)\\IamResponding\\IaR Autolauncher\\autoupdate.exe'
});
}

loadSelectBrowsers()
reviewInternetConnection()

async function reviewInternetConnection(){
    await internetAvailable({timeout: 15000, retries: 15,}).then(function(){
        logger.debug('INFO > renderer > CONNECTION > INTERNET AVAILABLE')
        document.getElementById('internetUnavailable').style.display = "none"
        start()   
    }).catch(function(){
        logger.debug('INFO > renderer > CONNECTION > NO CONNECTION')
        document.getElementById('internetUnavailable').style.display = "block"
    });
}

function start() {
    try {
        updateVersionFile()
        if (fs.existsSync('launch_data.txt')) {
            // read JSON object from file
            fs.readFile('launch_data.txt', 'utf-8', (err, data) => {
                if (err) {
                    logger.debug('ERROR > renderer > start > Reading user data in launch_data: ' + err)
                }
                if (data != undefined && data != '') {
                    // parse JSON object
                    var user;
                    try {
                        let buff = Buffer.from(data, 'base64');
                        let text = buff.toString('ascii');
                        var decoded = jwt.verify(text, ENCRYPT_DATA_SECRET_KEY);
                        var dcd = JSON.stringify(decoded)
                        user = JSON.parse(dcd)
                    } catch (err) {
                        logger.debug('ERROR > renderer > start > Decoding user data: ' + err)
                    }

                    const desencrypted_token = jwt_decode(user.token.token);

                    let _user = {}
                    _user.UserType = user.token.UserType
                    _user.token = user.token.token
                    _user.browser = user.token.browser
                    _user.agency = user.token.agency
                    _user.username = user.token.username
                    _user.pswd = user.token.pswd
                    _user.autolaunch = user.token.autolaunch
                    _user.enableKioskMode = user.token.enableKioskMode

                    logger.debug("INFO > renderer > getCookieBrowser > token > "+desencrypted_token.token);
                    ValidateSession(_user, desencrypted_token.token)
                } else {
                    logger.debug('INFO > renderer > start > The file launch.data.txt is empty or undefined.')
                }
            });
        } else {
            logger.debug('INFO > renderer > start > The file launch_data.txt not found.')
        }
    } catch (err) {
        logger.debug('ERROR > renderer > start > Reading file data: ' + err)
    }
}

function bytesToBase64(bytes) {
    const binString = String.fromCodePoint(...bytes);
    return btoa(binString);
}

async function ValidateSession(_user, token) {
    superagent
    .get(API_URL + '/Tokens')
    .ca(ca_cert)
    .set('Authorization', 'Token ' + token)
    .set('accept', 'json')
    .set('Accept-Charset', 'utf-8')
    .then( res => {
        if (res.statusCode == 200 && res.body == "1") {
            logger.debug('INFO > renderer > ValidateSession > API Get Token > Active Token')
            openUrl(_user);
            logLoginActivity(_user, 'launching active session');
        } else {
            logger.debug('INFO > renderer > ValidateSession > API Get Token > Inactive Token')
            createNewSession(_user)
        }
    })
    .catch(error => {
        logger.debug('ERROR > renderer > ValidateSession > ' + error)
    })
}
  
async function createNewSession(user) {
    const body= {
        OverrideSession: false,
        Username: user.username,
        Password: user.pswd,
        Agency: user.agency,
        AllowApparatusLogin: true,
        AutoLauncher: true
    }

    await superagent
    .post(API_URL + '/Tokens/')
    .ca(ca_cert)
    .send(body) // sends a JSON post body
    .set('accept', 'json')
    .then( res => {
        if (res.statusCode == 200) {
            logger.debug('INFO > renderer > createNewSession > Success')
            var encryptedToken = encrypt_module.encryptToken(res.body.Token)
            let usr = {}
            usr.browser = user.browser
            usr.token = encryptedToken
            usr.url = res.body.UserType == 3 ? IAR_MEMBER_URL : IAR_DISPATCHER_URL
            usr.agency = user.agency
            usr.username = user.username
            usr.pswd = user.pswd
            usr.autolaunch = user.autolaunch
            usr.acceptCookies = "true"
            usr.UserType = res.body.UserType
            usr.enableKioskMode = user.enableKioskMode

            saveUserData(usr)
            openUrl(usr);
            logLoginActivity(usr, 'updating session');

        } else {
            logger.debug('ERROR > renderer > createNewSession > Error create new session')
        }
    })
    .catch(error => {
        logger.debug('ERROR > renderer > createNewSession > ' + error)
    })
}
  
function openUrl(user) {
    // parse JSON object   
    logger.debug('INFO > renderer > openUrl > User Type: ' + user.UserType)
    
    var finalUrl = IAR_SECURITY + user.token
    logger.debug('INFO > renderer > openUrl > finalUrl to redirect: ' + finalUrl)

    try {
        ipc.send("hide-window");
        let command = ''
        let _browser = user.browser == "firefox" ? "Firefox" : "Google Chrome"
        let _finalBrowser = ""
        let counter = 0

        // get installed browsers
        detectBrowsers.getAvailableBrowsers()
            .then(browsers => {
                console.log('Browsers: ' + browsers)
                browsers.forEach(element => {
                    console.log('element.browser:' + element.browser)
                    if (element.browser == _browser) {
                        counter = counter + 1;
                    }
                });

                if (currentOS == 'linux') {
                    if (counter == 0 && _browser == "Firefox") {
                        _finalBrowser = "chromium-browser"
                    } else {
                        _finalBrowser = user.browser
                    }
                    if(user.enableKioskMode) {
                        command = _finalBrowser + ' --kiosk ' + finalUrl
                    } else {
                        command = _finalBrowser + ' ' + finalUrl
                    }
                } else // for windows
                {
                    if (counter == 0) {
                        _finalBrowser = _browser == "Firefox" ? "chrome" : "firefox"
                    } else {
                        _finalBrowser = user.browser
                    }
                    if(user.enableKioskMode) {
                        command = 'start ' + _finalBrowser + ' --kiosk ' + finalUrl
                    } else {
                        command = 'start ' + _finalBrowser + ' ' + finalUrl
                    }
                }
                childProcess.exec(command)
            })
    } catch (ex) {
        logger.debug('ERROR > renderer > openUrl > Opening the browser: ' + ex)
    }
}

async function loadSelectBrowsers() {
    var select = document.getElementById('ddlBrowser');
    const os = getOS(navigator.appVersion);

    if(os != 'none') {
        if(os == "linux") {
            var op = document.createElement('option');
            op.innerHTML = "Chromium";
            op.value = "chromium-browser";
            select.appendChild(op);
        }
    }

    // get installed browsers
    detectBrowsers.getAvailableBrowsers()
    .then(browsers => {
        browsers.forEach(element => {
            if (os != 'none') {
                var opt = document.createElement('option');
                if(os == "win" && element.browser == "Firefox") {           
                    opt.innerHTML = "Firefox";
                    opt.value = "firefox";
                    select.appendChild(opt);
                }else if (os == "win" && element.browser == "Google Chrome") {
                    opt.innerHTML = "Google Chrome";
                    opt.value = "chrome";
                    select.appendChild(opt);
                }else if(os == "linux" && element.browser == "Firefox") {
                    opt.innerHTML = "Firefox";
                    opt.value = "firefox";
                    select.appendChild(opt);
                }else if (os == "linux" && element.browser == "Google Chrome") {
                    opt.innerHTML = "Google Chrome";
                    opt.value = "chrome";
                    select.appendChild(opt);
                }
            }
        });
        getCookieLoginData()
    })
    .catch( error => logger.debug('ERROR > loadSelectBrowsers ' + error));
}

function logLoginActivity(_user, path) {

    let tokenBase64 = bytesToBase64(new TextEncoder().encode(_user.token));
    let macAddress = getMAC();
    let LOGS_URL = API_GATEWAY+"logs";

    logger.debug('INFO > renderer > logLoginActivity > ' + path + ' > ' + LOGS_URL)

    return superagent.post(LOGS_URL)
    .set("jwt-exchange", "yes")
    .set("Authorization", "Token" + tokenBase64)
    .send({
        "TraceId" : _user.username,
        "EventId" : "AutolauncherLogin",
        "Type" : 1,
        "Level" : null,
        "ErrorLevel" : null,
        "Message" : path,
        "Source" : "autolauncher",
        "TimeStamp": new Date(),
        "SentTimeStamp": new Date(),
        "AdditionalData" : [
            {
                "Key" : "Username",
                "Value" : _user.username
            },
            {
                "Key" : "Agency",
                "Value" : _user.agency
            },
            {
                "Key" : "InstallerVersion",
                "Value" : version
            },
            {
                "Key" : "MacAddress",
                "Value" : macAddress
            },
            {
                "Key" : "OS",
                "Value" : currentOS
            },
            {
                "Key" : "OSVersion",
                "Value" : os.release()
            },
            {
                "Key" : "Browser",
                "Value" : _user.browser
            }
        ]
    }).catch(error => {
        logger.debug('ERROR > renderer > logLoginActivity > ' + error)
    });
}

btnLaunch.addEventListener('click', function () {
    document.getElementById('internetUnavailable').style.display = "none"
    let selectedBrowser = document.getElementById('ddlBrowser').value
    
    if(cookieIsAccepted()) {
        if (document.getElementById('txtAgencyName').value == '') {
            openDialog('Agency Name is required.');
            modal.style.display = "block";
            return
        }
        if (document.getElementById('txtUsername').value == '') {
            openDialog('Username is required.')
            modal.style.display = "block"
            return
        }
        if (document.getElementById('txtPassword').value == '') {
            openDialog('Password is required.')
            modal.style.display = "block"
            return
        }
        if(document.getElementById('ddlBrowser').length == 0) {
            const os = getOS(navigator.appVersion);
            let message = os == "linux" ? "You must have at least one of the supported browsers installed. Try installing Firefox or Chromium." :
            "You must have at least one of the supported browsers installed. Try installing Firefox or Chrome.";
            openDialog(message);
            modal.style.display = "block";
            return
        }

        const body= {
            OverrideSession: false,
            Username: document.getElementById('txtUsername').value,
            Password: document.getElementById('txtPassword').value,
            Agency: document.getElementById('txtAgencyName').value,
            AllowApparatusLogin: true,
            AutoLauncher: true
        }

        superagent
        .post(API_URL + '/Tokens/')
        .ca(ca_cert)
        .send(body)
        .set('accept', 'json')
        .then( res => {
            if (res.statusCode == 200) {
                logger.debug('INFO > renderer > btnLaunch: Login success')

                //let json = JSON.parse(body)
                var encryptedToken = encrypt_module.encryptToken(res.body.Token)
                let usr = {}
                usr.browser = selectedBrowser
                usr.token = encryptedToken
                const set_autolaunch = document.getElementById('sliderOn').classList.contains('sliderHide') ? false : true
                usr.agency = document.getElementById('txtAgencyName').value
                usr.username = document.getElementById('txtUsername').value
                usr.pswd = document.getElementById('txtPassword').value
                usr.autolaunch = set_autolaunch
                usr.acceptCookies = "true"
                usr.UserType = res.body.UserType
                usr.url = res.body.UserType == 3 ? IAR_MEMBER_URL : IAR_DISPATCHER_URL
                usr.enableKioskMode = document.getElementById('onKiosk').classList.contains('sliderHide') ? false : true
                logger.debug('INFO > renderer > btnLaunch: User Type: ' + usr.UserType)

                saveUserData(usr);
                openUrl(usr);
                logLoginActivity(usr, 'creating new session');

                setAutolaunch(isAutoLauncher);
            } else {
                logger.debug('ERROR > renderer > btnLaunch > Error Login')
                openDialog('An error occurred, please try again.')
                modal.style.display = "block"
            }
        })
        .catch(error => {
            logger.debug('ERROR > renderer > btnLaunch > Error Login > ', error)
            openDialog('Invalid credentials')
            modal.style.display = "block"
        })
    }
}, false);

btnCancel.addEventListener('click', function () {
    ipc.send("close-window");
}, false);

closeModal.addEventListener('click', function () {
    modal.style.display = "none";
}, false);

function getOS(appVersion) {
    let name = "Not known";
    let idOS = "none";
    if (appVersion.indexOf("Win") != -1) {
        name = "Windows OS";
        idOS = "win";
    }
    else if (appVersion.indexOf("Linux") != -1) {
        name = "Linux OS";
        idOS = "linux";
    } else if (navigator.appVersion.indexOf("Mac") != -1) {
        Name = "Macintosh"
        idOS = "macOS"
    }
    else {
        name = navigator.appVersion
    }

    return idOS;
}

function openDialog(text) {
    document.getElementById("errorMessage").innerHTML = text;
}

function setAutolaunch(isAutolaunch) {
    if(currentOS == "win32") {
        logger.debug('INFO > renderer > SO: ' + currentOS)
        if(isAutolaunch) {
            logger.debug('INFO > renderer > Win startup enabled')
            autoupdateApp.enable()
        } else {
            logger.debug('INFO > renderer > Win startup desabled')
            autoupdateApp.disable()
        }
    }
}

sliderOff.addEventListener('click', function () {
    document.getElementById('sliderOn').classList.remove("sliderHide");
    document.getElementById('sliderOff').classList.add("sliderHide");
    isAutoLauncher = true;
}, false);

sliderOn.addEventListener('click', function() {
    document.getElementById('sliderOff').classList.remove("sliderHide");
    document.getElementById('sliderOn').classList.add("sliderHide");
    isAutoLauncher = false;
}, false);

function getCookieLoginData() {
    logger.debug('INFO > renderer > getCookieLoginData: Getting saved info')
    document.getElementById('year').innerHTML = new Date(new Date().toUTCString()).getUTCFullYear();

    try {
        if(fs.existsSync('launch_data.txt')) {
            logger.debug("INFO > renderer > The file launch_data exists.");

            // read JSON object from file
            fs.readFile('launch_data.txt', 'utf-8', (err, data) => {
                if (err) {
                    logger.debug('ERROR > renderer > Reading user data in launch_data: ' + err)
                }

                if(data != undefined && data != '') {
                    // parse JSON object
                    var user;
                    try {
                        let buff = Buffer.from(data, 'base64');
                        let text = buff.toString('ascii');
                        var decoded = jwt.verify(text, ENCRYPT_DATA_SECRET_KEY);

                        var dcd = JSON.stringify(decoded)
                        user = JSON.parse(dcd)
                    } catch(err) {
                        logger.debug('ERROR > renderer > Decoding user data: ' + err)
                    }

                    document.getElementById('txtAgencyName').value = user.token.agency
                    document.getElementById('txtUsername').value = user.token.username
                    document.getElementById('txtPassword').value = user.token.pswd
                    document.getElementById('ddlBrowser').value = user.token.browser
                    
                    let isAutolaunch = user.token.autolaunch
                    if(isAutolaunch) {
                        document.getElementById('sliderOn').classList.remove("sliderHide");
                        document.getElementById('sliderOff').classList.add("sliderHide");
                    } else {
                        document.getElementById('sliderOff').classList.remove("sliderHide");
                        document.getElementById('sliderOn').classList.add("sliderHide");
                    }

                    let isEnabledKioskMode = user.token.enableKioskMode
                    if(isEnabledKioskMode) {
                        document.getElementById('onKiosk').classList.remove("sliderHide");
                        document.getElementById('offKiosk').classList.add("sliderHide");
                    } else {
                        document.getElementById('offKiosk').classList.remove("sliderHide");
                        document.getElementById('onKiosk').classList.add("sliderHide");
                    }
                }
            });     
        } else {
            logger.debug('INFO > renderer > The file launch_data does not exist.');
        }
    } catch (err) {
        logger.debug('ERROR > renderer > Getting saved data.')
    }
    
    logger.debug('INFO > renderer > Is Cookie consent accepted? ' + cookieIsAccepted())
}

function saveUserData(data) {
    logger.debug('INFO > renderer > saveUserData > Saving user data.')
    var encrypted_data = encrypt_module.encryptData(data)

    // write JSON string to a file
    fs.writeFileSync('launch_data.txt', encrypted_data, (err) => {
        if (err) {
            logger.debug('ERROR > renderer > saveUserData > Saving user data: ' + err)
        }
        logger.debug("INFO > renderer > saveUserData > User data is saved.");
    });
}

document.getElementById('btnAcceptCookies').addEventListener('click', function () {
    document.getElementById('divCookies').style.display = "none"
    // document.getElementById('hdn').value = "true"

    const data = 'true'
    var encrypted_data = encrypt_module.encryptData(data)
    // write JSON string to a file
    fs.writeFileSync('terms.txt', encrypted_data, (err) => {
        if (err) {
            logger.debug('ERROR > renderer > Saving terms: ' + err)
        }
        logger.debug("INFO > renderer > Terms data are saved.");
    });
}, false);

function cookieIsAccepted() {
    let acceptCookies = false
    try {
        if(fs.existsSync('terms.txt')) {
            acceptCookies = true
            // read JSON object from file
            fs.readFile('terms.txt', 'utf-8', (err, data) => {
                if (err) {
                    logger.debug('ERROR > renderer > ' + err)
                }

                if(data != undefined && data != '') {
                    // parse JSON object
                    var terms;
                    try {
                        let buff = Buffer.from(data, 'base64');
                        let text = buff.toString('ascii');
                        var decoded = jwt.verify(text, ENCRYPT_DATA_SECRET_KEY);

                        var dcd = JSON.stringify(decoded)
                        terms = JSON.parse(dcd)
                    } catch(err) {
                        logger.debug('ERROR > renderer > Decoding terms data: ' + err)
                    }

                    if(terms.token == 'true') {
                        document.getElementById('divCookies').style.display = "none"
                        cookiesAccepted = true
                    } else {
                        document.getElementById('divCookies').style.display = "block"
                        cookiesAccepted = false
                    }
                }
            });     
        } else {
            document.getElementById('divCookies').style.display = "block"
        }
        return acceptCookies
    } catch (err) {
        logger.debug('ERROR > renderer > ' + err)
    }
}

document.getElementById('btnTermsOfUse').addEventListener('click', function () {
    shell.openExternal("https://www.iamresponding.com/terms-of-use/")
}, false);

offKiosk.addEventListener('click', function () {
    document.getElementById('onKiosk').classList.remove("sliderHide");
    document.getElementById('offKiosk').classList.add("sliderHide");
    enableKioskMode = true;
}, false);

onKiosk.addEventListener('click', function() {
    document.getElementById('offKiosk').classList.remove("sliderHide");
    document.getElementById('onKiosk').classList.add("sliderHide");
    enableKioskMode = false;
}, false);

function updateVersionFile() {
    //Add version (e.g. v1.0.0,0)
    fs.writeFile("IaRVersion.txt", version, (err) => {
        if (err) {
          logger.error('Error creating the file version:', err);
        }
    });
}

